/*
 * (C) Copyright 2011
 * CompuLab Ltd. <www.compulab.co.il>
 * Igor Grinberg <grinberg@compulab.co.il>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.
 */

#include <common.h>
#include <asm/arch/bits.h>
#include <asm/arch/mem.h>

#include "common.h"

/* Definitions for EMIF4 configuration values */
#define	EMIF4_TIM1_T_RP		0x3
#define	EMIF4_TIM1_T_RCD	0x3
#define	EMIF4_TIM1_T_WR		0x3
#define	EMIF4_TIM1_T_RAS	0x8
#define	EMIF4_TIM1_T_RC		0xA
#define	EMIF4_TIM1_T_RRD	0x2
#define	EMIF4_TIM1_T_WTR	0x2

#define	EMIF4_TIM2_T_XP		0x2
#define	EMIF4_TIM2_T_ODT	0x0
#define	EMIF4_TIM2_T_XSNR	0x1C
#define	EMIF4_TIM2_T_XSRD	0xC8
#define	EMIF4_TIM2_T_RTP	0x1
#define	EMIF4_TIM2_T_CKE	0x2

#define	EMIF4_TIM3_T_TDQSCKMAX	0x0
#define	EMIF4_TIM3_T_RFC	0x25
#define	EMIF4_TIM3_T_RAS_MAX	0x7

#define	EMIF4_PWR_IDLE		0x2
#define	EMIF4_PWR_DPD_EN	0x0
#define	EMIF4_PWR_PM_EN		0x0
#define	EMIF4_PWR_PM_TIM	0x0

#define	EMIF4_INITREF_DIS	0x0
#define	EMIF4_PASR		0x0
#define	EMIF4_REFRESH_RATE	0x50F

/* SDRAM Config register */
#define	EMIF4_CFG_SDRAM_TYP	0x2
#define	EMIF4_CFG_IBANK_POS	0x0
#define	EMIF4_CFG_DDR_TERM	0x0
#define	EMIF4_CFG_DDR2_DDQS	0x1
#define	EMIF4_CFG_DYN_ODT	0x0
#define	EMIF4_CFG_DDR_DIS_DLL	0x0
#define	EMIF4_CFG_SDR_DRV	0x0
#define	EMIF4_CFG_CWL		0x0
/* narrow mode is checked in run time and used for dram size detection */
#define	EMIF4_CFG_NARROW_MD	0x0
#define	EMIF4_CFG_CL		0x5
#define	EMIF4_CFG_ROWSIZE	0x0
#define	EMIF4_CFG_IBANK		0x3
#define	EMIF4_CFG_EBANK		0x0
#define	EMIF4_CFG_PGSIZE	0x2

/* EMIF4 PHY Control 1 register configuration */
#define EMIF4_DDR1_RD_LAT	0x6
#define EMIF4_DDR1_PWRDN_DIS	0x0
#define EMIF4_DDR1_STRBEN_EXT	0x0
#define EMIF4_DDR1_DLL_MODE	0x0
#define EMIF4_DDR1_VTP_DYN	0x0
#define EMIF4_DDR1_LB_CK_SEL	0x0

/* EMIF4 PHY Control 2 register configuration */
#define EMIF4_DDR2_TX_DATA_ALIGN	0x0
#define EMIF4_DDR2_RX_DLL_BYPASS	0x0

#define EMIF4_32_BIT_MODE	0
#define EMIF4_16_BIT_MODE	1

static void config_emif4(int narrow_mode)
{
	unsigned int regval;
	unsigned int emif4_cfg_narrow_md;

	emif4_cfg_narrow_md = narrow_mode == EMIF4_32_BIT_MODE ? 0 : 1;

	/* Set the DDR PHY parameters in PHY ctrl registers */
	regval = (EMIF4_DDR1_RD_LAT | (EMIF4_DDR1_PWRDN_DIS << 6) |
		 (EMIF4_DDR1_STRBEN_EXT << 7) | (EMIF4_DDR1_DLL_MODE << 12) |
		 (EMIF4_DDR1_VTP_DYN << 15) | (EMIF4_DDR1_LB_CK_SEL << 23));
	__raw_writel(regval, EMIF4_DDR_PHYCTL1);
	__raw_writel(regval, EMIF4_DDR_PHYCTL1_SHDW);

	regval = (EMIF4_DDR2_TX_DATA_ALIGN | (EMIF4_DDR2_RX_DLL_BYPASS << 1));
	__raw_writel(regval, EMIF4_DDR_PHYCTL2);

	/* Reset the DDR PHY and wait till completed */
	sr32(EMIF4_IODFT_TLGC, 10, 1, 1);
	/* Wait till that bit clears */
	while ((__raw_readl(EMIF4_IODFT_TLGC) & BIT10) == 0x1);
	/* Re-verify the DDR PHY status */
	while ((__raw_readl(EMIF4_SDRAM_STS) & BIT2) == 0x0);

	sr32(EMIF4_IODFT_TLGC, 0, 1, 1);
	/* Set SDR timing registers */
	regval = (EMIF4_TIM1_T_WTR | (EMIF4_TIM1_T_RRD << 3) |
		 (EMIF4_TIM1_T_RC << 6) | (EMIF4_TIM1_T_RAS << 12) |
		 (EMIF4_TIM1_T_WR << 17) | (EMIF4_TIM1_T_RCD << 21) |
		 (EMIF4_TIM1_T_RP << 25));
	__raw_writel(regval, EMIF4_SDRAM_TIM1);
	__raw_writel(regval, EMIF4_SDRAM_TIM1_SHDW);

	regval = (EMIF4_TIM2_T_CKE | (EMIF4_TIM2_T_RTP << 3) |
		 (EMIF4_TIM2_T_XSRD << 6) | (EMIF4_TIM2_T_XSNR << 16) |
		 (EMIF4_TIM2_T_ODT << 25) | (EMIF4_TIM2_T_XP << 28));
	__raw_writel(regval, EMIF4_SDRAM_TIM2);
	__raw_writel(regval, EMIF4_SDRAM_TIM2_SHDW);

	regval = (EMIF4_TIM3_T_RAS_MAX | (EMIF4_TIM3_T_RFC << 4) |
		 (EMIF4_TIM3_T_TDQSCKMAX << 13));
	__raw_writel(regval, EMIF4_SDRAM_TIM3);
	__raw_writel(regval, EMIF4_SDRAM_TIM3_SHDW);

	/* Set the PWR control register */
	regval = (EMIF4_PWR_PM_TIM | (EMIF4_PWR_PM_EN << 8) |
		 (EMIF4_PWR_DPD_EN << 10) | (EMIF4_PWR_IDLE << 30));
	__raw_writel(regval, EMIF4_PWR_MGT_CTRL);
	__raw_writel(regval, EMIF4_PWR_MGT_CTRL_SHDW);

	/* Set the DDR refresh rate control register */
	regval = (EMIF4_REFRESH_RATE | (EMIF4_PASR << 24) |
		 (EMIF4_INITREF_DIS << 31));
	__raw_writel(regval, EMIF4_SDRAM_RFCR);
	__raw_writel(regval, EMIF4_SDRAM_RFCR_SHDW);

	/* Set the SDRAM configuration register */
	regval = (EMIF4_CFG_PGSIZE | (EMIF4_CFG_EBANK << 3) |
		 (EMIF4_CFG_IBANK << 4) | (EMIF4_CFG_ROWSIZE << 7) |
		 (EMIF4_CFG_CL << 10) | (emif4_cfg_narrow_md << 14) |
		 (EMIF4_CFG_CWL << 16) | (EMIF4_CFG_SDR_DRV << 18) |
		 (EMIF4_CFG_DDR_DIS_DLL << 20) | (EMIF4_CFG_DYN_ODT << 21) |
		 (EMIF4_CFG_DDR2_DDQS << 23) | (EMIF4_CFG_DDR_TERM << 24) |
		 (EMIF4_CFG_IBANK_POS << 27) | (EMIF4_CFG_SDRAM_TYP << 29));
	__raw_writel(regval, EMIF4_SDRAM_CFG);
}

/********************************************************************
 * config_emif4_ddr() - Init/Configure DDR on EMIF4 controller.
 ********************************************************************/
#define DRAM_TEST_ADDRESS	0x80100000
#define DRAM_TEST_DUMMY		0x87654321
#define DRAM_TEST_VALUE		0x12345678

static void config_emif4_ddr(void)
{
	/* configure emif4 for 32 bit mode - 2 dram chips assembled */
	config_emif4(EMIF4_32_BIT_MODE);
	/*
	 * check if there are indeed 2 chips assembled
	 * write dummy check value, so there will be no false positives
	 */
	__raw_writel(DRAM_TEST_DUMMY, DRAM_TEST_ADDRESS);
	/* make sure write is complete */
	__raw_readl(DRAM_TEST_ADDRESS);
	/* write the actual check value */
	__raw_writel(DRAM_TEST_VALUE, DRAM_TEST_ADDRESS);

	if (__raw_readl(DRAM_TEST_ADDRESS) != DRAM_TEST_VALUE)
		config_emif4(EMIF4_16_BIT_MODE); /* 16 bit mode */
}

/*********************************************************************
 * setup_emif4_memory_controller() - init memory on boards with EMIF4.
 *********************************************************************/
void setup_emif4_memory_controller(void)
{
	/* enable the DDRPHY clk */
	sr32(OMAP34XX_CTRL_BASE + 0x588, 15, 15, 0x1);
	/* enable the EMIF4 clk */
	sr32(OMAP34XX_CTRL_BASE + 0x588, 14, 14, 0x1);

	/* Configure the EMIF4 for our DDR */
	config_emif4_ddr();
}

