/*
 * (C) Copyright 2011
 * CompuLab Ltd. <www.compulab.co.il>
 * Ilya Ledvich <ilya@compulab.co.il>
 * Igor Grinberg <grinberg@compulab.co.il>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.
 */

#include <common.h>
#include <asm/arch/bits.h>
#include <asm/arch/mem.h>

#include "common.h"

/********************************************************************
 * config_sdrc() - init mDDR
 ********************************************************************/
static void config_sdrc(unsigned long mdcfg)
{
	/* reset sdrc controller */
	__raw_writel(SOFTRESET, SDRC_SYSCONFIG);
	wait_on_value(BIT0, BIT0, SDRC_STATUS, 12000000);
	__raw_writel(0, SDRC_SYSCONFIG);

	/* setup sdrc to ball mux */
	__raw_writel(SDP_SDRC_SHARING, SDRC_SHARING);

	/* set mdcfg */
	__raw_writel(mdcfg, SDRC_MCFG_0);

	/* set timing */
	__raw_writel(MICRON_SDRC_ACTIM_CTRLA_0, SDRC_ACTIM_CTRLA_0);
	__raw_writel(MICRON_SDRC_ACTIM_CTRLB_0, SDRC_ACTIM_CTRLB_0);

	__raw_writel(SDP_SDRC_RFR_CTRL, SDRC_RFR_CTRL_0);
	__raw_writel(SDP_SDRC_POWER_POP, SDRC_POWER);

	/*
	 * init sequence for mDDR/mSDR using manual commands
	 * (DDR is different)
	 */
	__raw_writel(CMD_NOP, SDRC_MANUAL_0);
	delay(5000);
	__raw_writel(CMD_PRECHARGE, SDRC_MANUAL_0);
	__raw_writel(CMD_AUTOREFRESH, SDRC_MANUAL_0);
	__raw_writel(CMD_AUTOREFRESH, SDRC_MANUAL_0);

	/* set mr0 */
	__raw_writel(SDP_SDRC_MR_0_DDR, SDRC_MR_0);

	/* set up dll */
	__raw_writel(SDP_SDRC_DLLAB_CTRL, SDRC_DLLA_CTRL);
	delay(0x2000);	/* give time to lock */
}

/***********************************************************************
 * setup_sdrc_memory_controller() - init memory on boards with SDRC.
 ***********************************************************************/
void setup_sdrc_memory_controller(void)
{
	unsigned long size1, size2;
	unsigned long mdcfg;

	config_sdrc(SDRC_MDCFG_256MB_DDR);
	size1 = get_ram_size((volatile unsigned long *)DRAM_BASE, SZ_256M);

	config_sdrc(SDRC_MDCFG_64MB_DDR);
	size2 = get_ram_size((volatile unsigned long *)DRAM_BASE, SZ_64M);

	if (size2 > size1)
		mdcfg = SDRC_MDCFG_64MB_DDR;
	else if (size1 == SZ_256M)
		mdcfg = SDRC_MDCFG_256MB_DDR;
	else
		mdcfg = SDRC_MDCFG_128MB_DDR;

	config_sdrc(mdcfg);
}

